package sun.security.provider;

import sun.misc.SharedSecrets;
import java.util.HashMap;
import java.util.Random;
import java.util.Map;
import javax.security.auth.x500.X500Principal;
import java.security.cert.X509Certificate;
import java.util.List;
import javax.security.auth.Subject;
import java.util.Collections;
import java.util.HashSet;
import java.util.Arrays;
import java.security.Principal;
import java.security.Permissions;
import java.security.PermissionCollection;
import sun.misc.JavaSecurityProtectionDomainAccess;
import java.security.ProtectionDomain;
import java.util.ArrayList;
import java.security.KeyStoreException;
import java.util.StringTokenizer;
import java.security.AllPermission;
import java.net.NetPermission;
import java.net.SocketPermission;
import java.io.FilePermission;
import java.security.Permission;
import java.util.Iterator;
import java.lang.reflect.InvocationTargetException;
import java.security.UnresolvedPermission;
import java.net.MalformedURLException;
import java.util.PropertyPermission;
import sun.security.util.SecurityConstants;
import java.security.CodeSource;
import java.security.cert.Certificate;
import java.util.Enumeration;
import java.security.KeyStore;
import java.io.IOException;
import java.text.MessageFormat;
import sun.security.util.ResourcesMgr;
import java.io.InputStreamReader;
import sun.security.util.PolicyUtil;
import java.net.URI;
import sun.net.www.ParseUtil;
import java.io.File;
import sun.security.util.PropertyExpander;
import java.security.AccessController;
import java.security.Security;
import java.security.PrivilegedAction;
import java.net.URL;
import sun.security.util.Debug;
import java.security.Policy;

public class PolicyFile extends Policy
{
    private static final Debug debug;
    private static final String NONE = "NONE";
    private static final String P11KEYSTORE = "PKCS11";
    private static final String SELF = "${{self}}";
    private static final String X500PRINCIPAL = "javax.security.auth.x500.X500Principal";
    private static final String POLICY = "java.security.policy";
    private static final String SECURITY_MANAGER = "java.security.manager";
    private static final String POLICY_URL = "policy.url.";
    private static final String AUTH_POLICY = "java.security.auth.policy";
    private static final String AUTH_POLICY_URL = "auth.policy.url.";
    private static final int DEFAULT_CACHE_SIZE = 1;
    private volatile PolicyFile.PolicyInfo policyInfo;
    private boolean constructed;
    private boolean expandProperties;
    private boolean ignoreIdentityScope;
    private boolean allowSystemProperties;
    private boolean notUtf8;
    private URL url;
    private static final Class[] PARAMS0;
    private static final Class[] PARAMS1;
    private static final Class[] PARAMS2;
    
    public PolicyFile() {
        this.constructed = false;
        this.expandProperties = true;
        this.ignoreIdentityScope = true;
        this.allowSystemProperties = true;
        this.notUtf8 = false;
        this.init((URL)null);
    }
    
    public PolicyFile(URL url) {
        this.constructed = false;
        this.expandProperties = true;
        this.ignoreIdentityScope = true;
        this.allowSystemProperties = true;
        this.notUtf8 = false;
        this.init(this.url = url);
    }
    
    private void init(URL url) {
        String s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                PolicyFile.this.expandProperties = "true".equalsIgnoreCase(Security.getProperty("policy.expandProperties"));
                PolicyFile.this.ignoreIdentityScope = "true".equalsIgnoreCase(Security.getProperty("policy.ignoreIdentityScope"));
                PolicyFile.this.allowSystemProperties = "true".equalsIgnoreCase(Security.getProperty("policy.allowSystemProperty"));
                PolicyFile.this.notUtf8 = "false".equalsIgnoreCase(System.getProperty("sun.security.policy.utf8"));
                return System.getProperty("sun.security.policy.numcaches");
            }
        });
        int int1;
        if (s != null) {
            try {
                int1 = Integer.parseInt(s);
            }
            catch (NumberFormatException ex) {
                int1 = 1;
            }
        }
        else {
            int1 = 1;
        }
        PolicyFile.PolicyInfo policyInfo = new PolicyInfo(int1);
        this.initPolicyFile(policyInfo, url);
        this.policyInfo = policyInfo;
    }
    
    private void initPolicyFile(final PolicyFile.PolicyInfo policyInfo, final URL obj) {
        if (obj != null) {
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("reading " + obj);
            }
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    if (!PolicyFile.access$400(PolicyFile.this, obj, policyInfo)) {
                        PolicyFile.access$500(PolicyFile.this, policyInfo);
                    }
                    return null;
                }
            });
        }
        else {
            if (!this.initPolicyFile("java.security.policy", "policy.url.", policyInfo)) {
                this.initStaticPolicy(policyInfo);
            }
            this.initPolicyFile("java.security.auth.policy", "auth.policy.url.", policyInfo);
        }
    }
    
    private boolean initPolicyFile(final String s, final String s2, final PolicyFile.PolicyInfo policyInfo) {
        return (boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                boolean n = false;
                if (PolicyFile.this.allowSystemProperties) {
                    String value = System.getProperty(s);
                    if (value != null) {
                        boolean n2 = false;
                        if (value.startsWith("=")) {
                            n2 = true;
                            value = value.substring(1);
                        }
                        try {
                            String expand = PropertyExpander.expand(value);
                            File o = new File(expand);
                            URL fileToEncodedURL;
                            if (o.exists()) {
                                fileToEncodedURL = ParseUtil.fileToEncodedURL(new File(o.getCanonicalPath()));
                            }
                            else {
                                fileToEncodedURL = new URL(expand);
                            }
                            if (PolicyFile.debug != null) {
                                PolicyFile.debug.println("reading " + fileToEncodedURL);
                            }
                            if (PolicyFile.access$400(PolicyFile.this, fileToEncodedURL, policyInfo)) {
                                n = true;
                            }
                        }
                        catch (Exception obj) {
                            if (PolicyFile.debug != null) {
                                PolicyFile.debug.println("caught exception: " + obj);
                            }
                        }
                        if (!(n2 == 0)) {
                            if (PolicyFile.debug != null) {
                                PolicyFile.debug.println("overriding other policies!");
                            }
                            return Boolean.valueOf((boolean)n);
                        }
                    }
                }
                String property;
                for (int i = 1; (property = Security.getProperty(s2 + i)) != null; ++i) {
                    try {
                        String replace = PropertyExpander.expand(property).replace(File.separatorChar, '/');
                        URL obj2;
                        if (property.startsWith("file:${java.home}/") || property.startsWith("file:${user.home}/")) {
                            obj2 = new File(replace.substring(5)).toURI().toURL();
                        }
                        else {
                            obj2 = new URI(replace).toURL();
                        }
                        if (PolicyFile.debug != null) {
                            PolicyFile.debug.println("reading " + obj2);
                        }
                        if (PolicyFile.access$400(PolicyFile.this, obj2, policyInfo)) {
                            n = true;
                        }
                    }
                    catch (Exception obj3) {
                        if (PolicyFile.debug != null) {
                            PolicyFile.debug.println("Debug info only. Error reading policy " + obj3);
                            obj3.printStackTrace();
                        }
                    }
                }
                return Boolean.valueOf((boolean)n);
            }
        });
    }
    
    private boolean init(URL url, PolicyFile.PolicyInfo policyInfo) {
        boolean n = false;
        PolicyParser policyParser = new PolicyParser(this.expandProperties);
        InputStreamReader inputStreamReader = null;
        try {
            if (this.notUtf8) {
                inputStreamReader = new InputStreamReader(PolicyUtil.getInputStream(url));
            }
            else {
                inputStreamReader = new InputStreamReader(PolicyUtil.getInputStream(url), "UTF-8");
            }
            policyParser.read(inputStreamReader);
            KeyStore keyStore = null;
            try {
                keyStore = PolicyUtil.getKeyStore(url, policyParser.getKeyStoreUrl(), policyParser.getKeyStoreType(), policyParser.getKeyStoreProvider(), policyParser.getStorePassURL(), PolicyFile.debug);
            }
            catch (Exception ex) {
                if (PolicyFile.debug != null) {
                    PolicyFile.debug.println("Debug info only. Ignoring exception.");
                    ex.printStackTrace();
                }
            }
            Enumeration<PolicyParser.GrantEntry> grantElements = policyParser.grantElements();
            while (grantElements.hasMoreElements()) {
                this.addGrantEntry((PolicyParser.GrantEntry)grantElements.nextElement(), keyStore, policyInfo);
            }
        }
        catch (PolicyParser.ParsingException ex2) {
            System.err.println(new MessageFormat(ResourcesMgr.getString("java.security.policy.error.parsing.policy.message")).format(new Object[] { url, ex2.getLocalizedMessage() }));
            if (PolicyFile.debug != null) {
                ex2.printStackTrace();
            }
        }
        catch (Exception ex3) {
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("error parsing " + url);
                PolicyFile.debug.println(ex3.toString());
                ex3.printStackTrace();
            }
        }
        finally {
            if (inputStreamReader != null) {
                try {
                    inputStreamReader.close();
                    n = true;
                }
                catch (IOException ex4) {}
            }
            else {
                n = true;
            }
        }
        return n;
    }
    
    private void initStaticPolicy(final PolicyFile.PolicyInfo policyInfo) {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                PolicyFile.PolicyEntry policyEntry = new PolicyFile.PolicyEntry(new CodeSource(null, (Certificate[])null));
                policyEntry.add(SecurityConstants.LOCAL_LISTEN_PERMISSION);
                policyEntry.add(new PropertyPermission("java.version", "read"));
                policyEntry.add(new PropertyPermission("java.vendor", "read"));
                policyEntry.add(new PropertyPermission("java.vendor.url", "read"));
                policyEntry.add(new PropertyPermission("java.class.version", "read"));
                policyEntry.add(new PropertyPermission("os.name", "read"));
                policyEntry.add(new PropertyPermission("os.version", "read"));
                policyEntry.add(new PropertyPermission("os.arch", "read"));
                policyEntry.add(new PropertyPermission("file.separator", "read"));
                policyEntry.add(new PropertyPermission("path.separator", "read"));
                policyEntry.add(new PropertyPermission("line.separator", "read"));
                policyEntry.add(new PropertyPermission("java.specification.version", "read"));
                policyEntry.add(new PropertyPermission("java.specification.vendor", "read"));
                policyEntry.add(new PropertyPermission("java.specification.name", "read"));
                policyEntry.add(new PropertyPermission("java.vm.specification.version", "read"));
                policyEntry.add(new PropertyPermission("java.vm.specification.vendor", "read"));
                policyEntry.add(new PropertyPermission("java.vm.specification.name", "read"));
                policyEntry.add(new PropertyPermission("java.vm.version", "read"));
                policyEntry.add(new PropertyPermission("java.vm.vendor", "read"));
                policyEntry.add(new PropertyPermission("java.vm.name", "read"));
                policyInfo.policyEntries.add(policyEntry);
                String[] extDirs = PolicyParser.parseExtDirs("${{java.ext.dirs}}", 0);
                if (extDirs != null && extDirs.length > 0) {
                    for (int i = 0; i < extDirs.length; ++i) {
                        try {
                            PolicyFile.PolicyEntry o = new PolicyFile.PolicyEntry(PolicyFile.access$700(PolicyFile.this, new CodeSource(new URL(extDirs[i]), (Certificate[])null), false));
                            o.add(SecurityConstants.ALL_PERMISSION);
                            policyInfo.policyEntries.add(o);
                        }
                        catch (Exception ex) {}
                    }
                }
                return null;
            }
        });
    }
    
    private CodeSource getCodeSource(PolicyParser.GrantEntry grantEntry, KeyStore keyStore, PolicyFile.PolicyInfo policyInfo) throws MalformedURLException {
        Certificate[] certificates = null;
        if (grantEntry.signedBy != null) {
            certificates = this.getCertificates(keyStore, grantEntry.signedBy, policyInfo);
            if (certificates == null) {
                if (PolicyFile.debug != null) {
                    PolicyFile.debug.println("  -- No certs for alias '" + grantEntry.signedBy + "' - ignoring entry");
                }
                return null;
            }
        }
        Object url;
        if (grantEntry.codeBase != null) {
            url = new URL(grantEntry.codeBase);
        }
        else {
            url = null;
        }
        return this.canonicalizeCodebase(new CodeSource(url, certificates), false);
    }
    
    private void addGrantEntry(PolicyParser.GrantEntry grantEntry, KeyStore keyStore, PolicyFile.PolicyInfo policyInfo) {
        if (PolicyFile.debug != null) {
            PolicyFile.debug.println("Adding policy entry: ");
            PolicyFile.debug.println("  signedBy " + grantEntry.signedBy);
            PolicyFile.debug.println("  codeBase " + grantEntry.codeBase);
            if (grantEntry.principals != null) {
                Iterator<Object> iterator = grantEntry.principals.iterator();
                while (iterator.hasNext()) {
                    PolicyFile.debug.println("  " + ((PolicyParser.PrincipalEntry)iterator.next()).toString());
                }
            }
        }
        try {
            CodeSource codeSource = this.getCodeSource(grantEntry, keyStore, policyInfo);
            if (codeSource == null) {
                return;
            }
            if (!this.replacePrincipals(grantEntry.principals, keyStore)) {
                return;
            }
            PolicyFile.PolicyEntry o = new PolicyFile.PolicyEntry(codeSource, grantEntry.principals);
            Enumeration<PolicyParser.PermissionEntry> permissionElements = grantEntry.permissionElements();
            while (permissionElements.hasMoreElements()) {
                PolicyParser.PermissionEntry permissionEntry = (PolicyParser.PermissionEntry)permissionElements.nextElement();
                try {
                    this.expandPermissionName(permissionEntry, keyStore);
                    if (permissionEntry.permission.equals("javax.security.auth.PrivateCredentialPermission") && permissionEntry.name.endsWith(" self")) {
                        permissionEntry.name = permissionEntry.name.substring(0, permissionEntry.name.indexOf("self")) + "${{self}}";
                    }
                    Object instance;
                    if (permissionEntry.name != null && permissionEntry.name.indexOf("${{self}}") != -1) {
                        Certificate[] certificates;
                        if (permissionEntry.signedBy != null) {
                            certificates = this.getCertificates(keyStore, permissionEntry.signedBy, policyInfo);
                        }
                        else {
                            certificates = null;
                        }
                        instance = new PolicyFile.SelfPermission(permissionEntry.permission, permissionEntry.name, permissionEntry.action, certificates);
                    }
                    else {
                        instance = getInstance(permissionEntry.permission, permissionEntry.name, permissionEntry.action);
                    }
                    o.add(instance);
                    if (PolicyFile.debug != null) {
                        PolicyFile.debug.println("  " + instance);
                    }
                    else {
                        continue;
                    }
                }
                catch (ClassNotFoundException ex) {
                    Certificate[] certificates2;
                    if (permissionEntry.signedBy != null) {
                        certificates2 = this.getCertificates(keyStore, permissionEntry.signedBy, policyInfo);
                    }
                    else {
                        certificates2 = null;
                    }
                    if (certificates2 != null || permissionEntry.signedBy == null) {
                        UnresolvedPermission obj = new UnresolvedPermission(permissionEntry.permission, permissionEntry.name, permissionEntry.action, certificates2);
                        o.add(obj);
                        if (PolicyFile.debug != null) {
                            PolicyFile.debug.println("  " + obj);
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
                catch (InvocationTargetException ex2) {
                    System.err.println(new MessageFormat(ResourcesMgr.getString("java.security.policy.error.adding.Permission.perm.message")).format(new Object[] { permissionEntry.permission, ex2.getTargetException().toString() }));
                }
                catch (Exception ex3) {
                    System.err.println(new MessageFormat(ResourcesMgr.getString("java.security.policy.error.adding.Permission.perm.message")).format(new Object[] { permissionEntry.permission, ex3.toString() }));
                }
            }
            policyInfo.policyEntries.add(o);
        }
        catch (Exception ex4) {
            System.err.println(new MessageFormat(ResourcesMgr.getString("java.security.policy.error.adding.Entry.message")).format(new Object[] { ex4.toString() }));
        }
        if (PolicyFile.debug != null) {
            PolicyFile.debug.println();
        }
    }
    
    private static final Permission getInstance(String s, String s2, String s3) throws ClassNotFoundException, InstantiationException, IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        Class<Object> forName = Class.forName(s, false, null);
        Permission knownInstance = getKnownInstance(forName, s2, s3);
        if (knownInstance != null) {
            return knownInstance;
        }
        if (!Permission.class.isAssignableFrom(forName)) {
            throw new ClassCastException(s + " is not a Permission");
        }
        if (s2 == null && s3 == null) {
            try {
                return (Permission)forName.getConstructor(PolicyFile.PARAMS0).newInstance(new Object[0]);
            }
            catch (NoSuchMethodException ex) {
                try {
                    return (Permission)forName.getConstructor(PolicyFile.PARAMS1).newInstance(new Object[] { s2 });
                }
                catch (NoSuchMethodException ex2) {
                    return (Permission)forName.getConstructor(PolicyFile.PARAMS2).newInstance(new Object[] { s2, s3 });
                }
            }
        }
        if (s2 != null && s3 == null) {
            try {
                return (Permission)forName.getConstructor(PolicyFile.PARAMS1).newInstance(new Object[] { s2 });
            }
            catch (NoSuchMethodException ex3) {
                return (Permission)forName.getConstructor(PolicyFile.PARAMS2).newInstance(new Object[] { s2, s3 });
            }
        }
        else {
            return (Permission)forName.getConstructor(PolicyFile.PARAMS2).newInstance(new Object[] { s2, s3 });
        }
    }
    
    private static final Permission getKnownInstance(Class<?> clazz, String path, String actions) {
        if (clazz.equals(FilePermission.class)) {
            return new FilePermission(path, actions);
        }
        if (clazz.equals(SocketPermission.class)) {
            return new SocketPermission(path, actions);
        }
        if (clazz.equals(RuntimePermission.class)) {
            return new RuntimePermission(path, actions);
        }
        if (clazz.equals(PropertyPermission.class)) {
            return new PropertyPermission(path, actions);
        }
        if (clazz.equals(NetPermission.class)) {
            return new NetPermission(path, actions);
        }
        if (clazz.equals(AllPermission.class)) {
            return SecurityConstants.ALL_PERMISSION;
        }
        return null;
    }
    
    private Certificate[] getCertificates(KeyStore keyStore, String str, PolicyFile.PolicyInfo policyInfo) {
        Object o = null;
        StringTokenizer stringTokenizer = new StringTokenizer(str, ",");
        int n = 0;
        while (stringTokenizer.hasMoreTokens()) {
            String trim = stringTokenizer.nextToken().trim();
            ++n;
            Certificate certificate = null;
            synchronized (policyInfo.aliasMapping) {
                certificate = (Certificate)policyInfo.aliasMapping.get(trim);
                if (certificate == null && keyStore != null) {
                    try {
                        certificate = keyStore.getCertificate(trim);
                    }
                    catch (KeyStoreException ex) {}
                    if (certificate != null) {
                        policyInfo.aliasMapping.put(trim, certificate);
                        policyInfo.aliasMapping.put(certificate, trim);
                    }
                }
            }
            if (certificate != null) {
                if (o == null) {
                    o = new ArrayList();
                }
                o.add(certificate);
            }
            else {
                continue;
            }
        }
        if (o != null && n == o.size()) {
            Certificate[] array = new Certificate[o.size()];
            o.toArray(array);
            return array;
        }
        return null;
    }
    
    @Override
    public void refresh() {
        this.init(this.url);
    }
    
    @Override
    public boolean implies(ProtectionDomain protectionDomain, Permission permission) {
        JavaSecurityProtectionDomainAccess.ProtectionDomainCache pdMapping = this.policyInfo.getPdMapping();
        PermissionCollection value = pdMapping.get(protectionDomain);
        if (value != null) {
            return value.implies(permission);
        }
        PermissionCollection permissions = this.getPermissions(protectionDomain);
        if (permissions == null) {
            return false;
        }
        pdMapping.put(protectionDomain, permissions);
        return permissions.implies(permission);
    }
    
    @Override
    public PermissionCollection getPermissions(ProtectionDomain protectionDomain) {
        Permissions permissions = new Permissions();
        if (protectionDomain == null) {
            return permissions;
        }
        this.getPermissions(permissions, protectionDomain);
        PermissionCollection permissions2 = protectionDomain.getPermissions();
        if (permissions2 != null) {
            synchronized (permissions2) {
                Enumeration<Permission> elements = permissions2.elements();
                while (elements.hasMoreElements()) {
                    permissions.add((Permission)elements.nextElement());
                }
            }
        }
        return permissions;
    }
    
    @Override
    public PermissionCollection getPermissions(CodeSource codeSource) {
        return this.getPermissions(new Permissions(), codeSource);
    }
    
    private PermissionCollection getPermissions(Permissions permissions, ProtectionDomain protectionDomain) {
        if (PolicyFile.debug != null) {
            PolicyFile.debug.println("getPermissions:\n\t" + this.printPD(protectionDomain));
        }
        final CodeSource codeSource = protectionDomain.getCodeSource();
        if (codeSource == null) {
            return permissions;
        }
        return this.getPermissions(permissions, (CodeSource)AccessController.doPrivileged(new PrivilegedAction<CodeSource>() {
            @Override
            public CodeSource run() {
                return PolicyFile.access$700(PolicyFile.this, codeSource, true);
            }
        }), protectionDomain.getPrincipals());
    }
    
    private PermissionCollection getPermissions(Permissions permissions, final CodeSource codeSource) {
        if (codeSource == null) {
            return permissions;
        }
        return this.getPermissions(permissions, (CodeSource)AccessController.doPrivileged(new PrivilegedAction<CodeSource>() {
            @Override
            public CodeSource run() {
                return PolicyFile.access$700(PolicyFile.this, codeSource, true);
            }
        }), (Principal[])null);
    }
    
    private Permissions getPermissions(Permissions permissions, CodeSource codeSource, Principal[] array) {
        PolicyFile.PolicyInfo policyInfo = this.policyInfo;
        Iterator<PolicyFile.PolicyEntry> iterator = policyInfo.policyEntries.iterator();
        while (iterator.hasNext()) {
            this.addPermissions(permissions, codeSource, array, (PolicyFile.PolicyEntry)iterator.next());
        }
        synchronized (policyInfo.identityPolicyEntries) {
            Iterator<PolicyFile.PolicyEntry> iterator2 = policyInfo.identityPolicyEntries.iterator();
            while (iterator2.hasNext()) {
                this.addPermissions(permissions, codeSource, array, (PolicyFile.PolicyEntry)iterator2.next());
            }
        }
        if (!this.ignoreIdentityScope) {
            Certificate[] certificates = codeSource.getCertificates();
            if (certificates != null) {
                for (int i = 0; i < certificates.length; ++i) {
                    if (policyInfo.aliasMapping.get(certificates[i]) == null && this.checkForTrustedIdentity(certificates[i], policyInfo)) {
                        permissions.add(SecurityConstants.ALL_PERMISSION);
                    }
                }
            }
        }
        return permissions;
    }
    
    private void addPermissions(Permissions permissions, final CodeSource obj, Principal[] a, final PolicyFile.PolicyEntry policyEntry) {
        if (PolicyFile.debug != null) {
            PolicyFile.debug.println("evaluate codesources:\n\tPolicy CodeSource: " + policyEntry.getCodeSource() + "\n\tActive CodeSource: " + obj);
        }
        if (!(boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                return new Boolean(policyEntry.getCodeSource().implies(obj));
            }
        })) {
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("evaluation (codesource) failed");
            }
            return;
        }
        else {
            List<PolicyParser.PrincipalEntry> principals = policyEntry.getPrincipals();
            if (PolicyFile.debug != null) {
                ArrayList<PolicyParser.PrincipalEntry> obj2 = new ArrayList();
                if (a != null) {
                    for (int i = 0; i < a.length; ++i) {
                        obj2.add(new PolicyParser.PrincipalEntry(a[i].getClass().getName(), a[i].getName()));
                    }
                }
                PolicyFile.debug.println("evaluate principals:\n\tPolicy Principals: " + principals + "\n\tActive Principals: " + obj2);
            }
            if (principals == null || principals.isEmpty()) {
                this.addPerms(permissions, a, policyEntry);
                if (PolicyFile.debug != null) {
                    PolicyFile.debug.println("evaluation (codesource/principals) passed");
                }
                return;
            }
            else if (a == null || a.length == 0) {
                if (PolicyFile.debug != null) {
                    PolicyFile.debug.println("evaluation (principals) failed");
                }
                return;
            }
            else {
                for (PolicyParser.PrincipalEntry principalEntry : principals) {
                    if (principalEntry.isWildcardClass()) {
                        continue;
                    }
                    if (principalEntry.isWildcardName()) {
                        if (wildcardPrincipalNameImplies(principalEntry.principalClass, a)) {
                            continue;
                        }
                        if (PolicyFile.debug != null) {
                            PolicyFile.debug.println("evaluation (principal name wildcard) failed");
                        }
                        return;
                    }
                    else {
                        Subject o = new Subject(true, new HashSet(Arrays.asList(a)), Collections.EMPTY_SET, Collections.EMPTY_SET);
                        try {
                            Class<Object> forName = Class.forName(principalEntry.principalClass, false, Thread.currentThread().getContextClassLoader());
                            if (!Principal.class.isAssignableFrom(forName)) {
                                throw new ClassCastException(principalEntry.principalClass + " is not a Principal");
                            }
                            Principal principal = (Principal)forName.getConstructor(PolicyFile.PARAMS1).newInstance(new Object[] { principalEntry.principalName });
                            if (PolicyFile.debug != null) {
                                PolicyFile.debug.println("found Principal " + principal.getClass().getName());
                            }
                            if (!principal.implies(o)) {
                                if (PolicyFile.debug != null) {
                                    PolicyFile.debug.println("evaluation (principal implies) failed");
                                }
                                return;
                            }
                            else {
                                continue;
                            }
                        }
                        catch (Exception ex) {
                            if (PolicyFile.debug != null) {
                                ex.printStackTrace();
                            }
                            if (!principalEntry.implies(o)) {
                                if (PolicyFile.debug != null) {
                                    PolicyFile.debug.println("evaluation (default principal implies) failed");
                                }
                                return;
                            }
                            else {
                                continue;
                            }
                        }
                    }
                }
                if (PolicyFile.debug != null) {
                    PolicyFile.debug.println("evaluation (codesource/principals) passed");
                }
                this.addPerms(permissions, a, policyEntry);
                return;
            }
        }
    }
    
    private static boolean wildcardPrincipalNameImplies(String s, Principal[] array) {
        int length = array.length;
        int i = 0;
        while (i < length) {
            if (s.equals(array[i].getClass().getName())) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private void addPerms(Permissions permissions, Principal[] array, PolicyFile.PolicyEntry policyEntry) {
        for (int i = 0; i < policyEntry.permissions.size(); ++i) {
            Permission permission = (Permission)policyEntry.permissions.get(i);
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("  granting " + permission);
            }
            if ((permission instanceof PolicyFile.SelfPermission)) {
                this.expandSelf((PolicyFile.SelfPermission)permission, policyEntry.getPrincipals(), array, permissions);
            }
            else {
                permissions.add(permission);
            }
        }
    }
    
    private void expandSelf(PolicyFile.SelfPermission selfPermission, List<PolicyParser.PrincipalEntry> list, Principal[] array, Permissions permissions) {
        if (list == null || list.isEmpty()) {
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("Ignoring permission " + selfPermission.getSelfType() + " with target name (" + selfPermission.getSelfName() + ").  No Principal(s) specified in the grant clause.  SELF-based target names are only valid in the context of a Principal-based grant entry.");
            }
            return;
        }
        else {
            int beginIndex = 0;
            StringBuilder sb = new StringBuilder();
            int index;
            while ((index = selfPermission.getSelfName().indexOf("${{self}}", beginIndex)) != -1) {
                sb.append(selfPermission.getSelfName().substring(beginIndex, index));
                Iterator<Object> iterator = list.iterator();
                while (iterator.hasNext()) {
                    String[][] principalInfo = this.getPrincipalInfo((PolicyParser.PrincipalEntry)iterator.next(), array);
                    for (int i = 0; i < principalInfo.length; ++i) {
                        if (i != 0) {
                            sb.append(", ");
                        }
                        sb.append(principalInfo[i][0] + " \"" + principalInfo[i][1] + "\"");
                    }
                    if (iterator.hasNext()) {
                        sb.append(", ");
                    }
                    else {
                        continue;
                    }
                }
                beginIndex = index + "${{self}}".length();
            }
            sb.append(selfPermission.getSelfName().substring(beginIndex));
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("  expanded:\n\t" + selfPermission.getSelfName() + "\n  into:\n\t" + sb.toString());
            }
            try {
                permissions.add(getInstance(selfPermission.getSelfType(), sb.toString(), selfPermission.getSelfActions()));
            }
            catch (ClassNotFoundException ex) {
                Class<Object> class1 = null;
                synchronized (permissions) {
                    Enumeration<Permission> elements = permissions.elements();
                    while (elements.hasMoreElements()) {
                        Permission permission = (Permission)elements.nextElement();
                        if (permission.getClass().getName().equals(selfPermission.getSelfType())) {
                            class1 = permission.getClass();
                            break;
                        }
                        continue;
                    }
                }
                if (class1 == null) {
                    permissions.add(new UnresolvedPermission(selfPermission.getSelfType(), sb.toString(), selfPermission.getSelfActions(), selfPermission.getCerts()));
                }
                else {
                    try {
                        if (selfPermission.getSelfActions() == null) {
                            try {
                                permissions.add((Permission)class1.getConstructor(PolicyFile.PARAMS1).newInstance(new Object[] { sb.toString() }));
                            }
                            catch (NoSuchMethodException ex2) {
                                permissions.add((Permission)class1.getConstructor(PolicyFile.PARAMS2).newInstance(new Object[] { sb.toString(), selfPermission.getSelfActions() }));
                            }
                        }
                        else {
                            permissions.add((Permission)class1.getConstructor(PolicyFile.PARAMS2).newInstance(new Object[] { sb.toString(), selfPermission.getSelfActions() }));
                        }
                    }
                    catch (Exception ex3) {
                        if (PolicyFile.debug != null) {
                            PolicyFile.debug.println("self entry expansion  instantiation failed: " + ex3.toString());
                        }
                    }
                }
            }
            catch (Exception ex4) {
                if (PolicyFile.debug != null) {
                    PolicyFile.debug.println(ex4.toString());
                }
            }
            return;
        }
    }
    
    private String[][] getPrincipalInfo(PolicyParser.PrincipalEntry principalEntry, Principal[] array) {
        if (!principalEntry.isWildcardClass() && !principalEntry.isWildcardName()) {
            String[][] array4 = new String[1][2];
            array4[0][0] = principalEntry.principalClass;
            array4[0][1] = principalEntry.principalName;
            return array4;
        }
        if (!principalEntry.isWildcardClass() && principalEntry.isWildcardName()) {
            ArrayList<Object> list = new ArrayList();
            for (int j = 0; j < array.length; ++j) {
                if (principalEntry.principalClass.equals(array[j].getClass().getName())) {
                    list.add(array[j]);
                }
            }
            String[][] array3 = new String[list.size()][2];
            int n = 0;
            for (Principal principal : list) {
                array3[n][0] = principal.getClass().getName();
                array3[n][1] = principal.getName();
                ++n;
            }
            return array3;
        }
        String[][] array2 = new String[array.length][2];
        for (int i = 0; i < array.length; ++i) {
            array2[i][0] = array[i].getClass().getName();
            array2[i][1] = array[i].getName();
        }
        return array2;
    }
    
    protected Certificate[] getSignerCertificates(CodeSource codeSource) {
        Certificate[] certificates;
        if ((certificates = codeSource.getCertificates()) == null) {
            return null;
        }
        int i = 0;
        while (i < certificates.length) {
            if (!(certificates[i] instanceof X509Certificate)) {
                return codeSource.getCertificates();
            }
            ++i;
        }
        int j = 0;
        int n = 0;
        while (j < certificates.length) {
            ++n;
            while (j + 1 < certificates.length && ((X509Certificate)certificates[j]).getIssuerDN().equals(((X509Certificate)certificates[j + 1]).getSubjectDN())) {
                ++j;
            }
            ++j;
        }
        if (n == certificates.length) {
            return certificates;
        }
        ArrayList<Object> list = new ArrayList();
        for (int k = 0; k < certificates.length; ++k) {
            list.add(certificates[k]);
            while (k + 1 < certificates.length && ((X509Certificate)certificates[k]).getIssuerDN().equals(((X509Certificate)certificates[k + 1]).getSubjectDN())) {
                ++k;
            }
        }
        Certificate[] array = new Certificate[list.size()];
        list.toArray(array);
        return array;
    }
    
    private CodeSource canonicalizeCodebase(CodeSource o, boolean b) {
        String decode = null;
        URL location = o.getLocation();
        if (location != null) {
            if (location.getProtocol().equals("jar")) {
                String file = location.getFile();
                int index = file.indexOf("!/");
                if (index != -1) {
                    try {
                        location = new URL(file.substring(0, index));
                    }
                    catch (MalformedURLException ex) {}
                }
            }
            if (location.getProtocol().equals("file")) {
                String host = location.getHost();
                if (host == null || host.equals("") || host.equals("~") || host.equalsIgnoreCase("localhost")) {
                    decode = ParseUtil.decode(location.getFile().replace('/', File.separatorChar));
                }
            }
        }
        if (decode != null) {
            try {
                URL fileToEncodedURL = ParseUtil.fileToEncodedURL(new File(canonPath(decode)));
                if (b) {
                    o = new CodeSource(fileToEncodedURL, this.getSignerCertificates(o));
                }
                else {
                    o = new CodeSource(fileToEncodedURL, o.getCertificates());
                }
            }
            catch (IOException ex2) {
                if (b) {
                    o = new CodeSource(o.getLocation(), this.getSignerCertificates(o));
                }
            }
        }
        else if (b) {
            o = new CodeSource(o.getLocation(), this.getSignerCertificates(o));
        }
        return o;
    }
    
    private static String canonPath(String s) throws IOException {
        if (s.endsWith("*")) {
            s = s.substring(0, s.length() - 1) + "-";
            s = new File(s).getCanonicalPath();
            return s.substring(0, s.length() - 1) + "*";
        }
        return new File(s).getCanonicalPath();
    }
    
    private String printPD(ProtectionDomain protectionDomain) {
        Principal[] principals = protectionDomain.getPrincipals();
        String string = "<no principals>";
        if (principals != null && principals.length > 0) {
            StringBuilder sb = new StringBuilder("(principals ");
            for (int i = 0; i < principals.length; ++i) {
                sb.append(principals[i].getClass().getName() + " \"" + principals[i].getName() + "\"");
                if (i < principals.length - 1) {
                    sb.append(", ");
                }
                else {
                    sb.append(")");
                }
            }
            string = sb.toString();
        }
        return "PD CodeSource: " + protectionDomain.getCodeSource() + "\n\tPD ClassLoader: " + protectionDomain.getClassLoader() + "\n\tPD Principals: " + string;
    }
    
    private boolean replacePrincipals(List<PolicyParser.PrincipalEntry> list, KeyStore keyStore) {
        if (list == null || list.isEmpty() || keyStore == null) {
            return true;
        }
        for (PolicyParser.PrincipalEntry principalEntry : list) {
            if (principalEntry.isReplaceName()) {
                String dn;
                if ((dn = this.getDN(principalEntry.principalName, keyStore)) == null) {
                    return false;
                }
                if (PolicyFile.debug != null) {
                    PolicyFile.debug.println("  Replacing \"" + principalEntry.principalName + "\" with " + "javax.security.auth.x500.X500Principal" + "/\"" + dn + "\"");
                }
                principalEntry.principalClass = "javax.security.auth.x500.X500Principal";
                principalEntry.principalName = dn;
            }
            else {
                continue;
            }
        }
        return true;
    }
    
    private void expandPermissionName(PolicyParser.PermissionEntry permissionEntry, KeyStore keyStore) throws Exception {
        if (permissionEntry.name == null || permissionEntry.name.indexOf("${{", 0) == -1) {
            return;
        }
        int beginIndex = 0;
        StringBuilder sb = new StringBuilder();
        int index;
        while ((index = permissionEntry.name.indexOf("${{", beginIndex)) != -1) {
            int index2 = permissionEntry.name.indexOf("}}", index);
            if (index2 < 1) {
                break;
            }
            sb.append(permissionEntry.name.substring(beginIndex, index));
            String s = permissionEntry.name.substring(index + 3, index2);
            int index3;
            if ((index3 = s.indexOf(":")) != -1) {
                s = s.substring(0, index3);
            }
            if (s.equalsIgnoreCase("self")) {
                sb.append(permissionEntry.name.substring(index, index2 + 2));
                beginIndex = index2 + 2;
            }
            else if (s.equalsIgnoreCase("alias")) {
                if (index3 == -1) {
                    throw new Exception(new MessageFormat(ResourcesMgr.getString("alias.name.not.provided.pe.name.")).format(new Object[] { permissionEntry.name }));
                }
                String dn;
                if ((dn = this.getDN(s.substring(index3 + 1), keyStore)) == null) {
                    throw new Exception(new MessageFormat(ResourcesMgr.getString("unable.to.perform.substitution.on.alias.suffix")).format(new Object[] { s.substring(index3 + 1) }));
                }
                sb.append("javax.security.auth.x500.X500Principal \"" + dn + "\"");
                beginIndex = index2 + 2;
            }
            else {
                throw new Exception(new MessageFormat(ResourcesMgr.getString("substitution.value.prefix.unsupported")).format(new Object[] { s }));
            }
        }
        sb.append(permissionEntry.name.substring(beginIndex));
        if (PolicyFile.debug != null) {
            PolicyFile.debug.println("  Permission name expanded from:\n\t" + permissionEntry.name + "\nto\n\t" + sb.toString());
        }
        permissionEntry.name = sb.toString();
        return;
    }
    
    private String getDN(String str, KeyStore keyStore) {
        Certificate certificate;
        try {
            certificate = keyStore.getCertificate(str);
        }
        catch (Exception ex) {
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("  Error retrieving certificate for '" + str + "': " + ex.toString());
            }
            return null;
        }
        if (certificate == null || !(certificate instanceof X509Certificate)) {
            if (PolicyFile.debug != null) {
                PolicyFile.debug.println("  -- No certificate for '" + str + "' - ignoring entry");
            }
            return null;
        }
        else {
            return new X500Principal(((X509Certificate)certificate).getSubjectX500Principal().toString()).getName();
        }
    }
    
    private boolean checkForTrustedIdentity(Certificate certificate, PolicyFile.PolicyInfo policyInfo) {
        return false;
    }
    
    static boolean access$400(PolicyFile policyFile, URL url, PolicyFile.PolicyInfo policyInfo) {
        return policyFile.init(url, policyInfo);
    }
    
    static void access$500(PolicyFile policyFile, PolicyFile.PolicyInfo policyInfo) {
        policyFile.initStaticPolicy(policyInfo);
    }
    
    static CodeSource access$700(PolicyFile policyFile, CodeSource codeSource, boolean b) {
        return policyFile.canonicalizeCodebase(codeSource, b);
    }
    
    static {
        debug = Debug.getInstance("policy");
        PARAMS0 = new Class[0];
        PARAMS1 = new Class[] { String.class };
        PARAMS2 = new Class[] { String.class, String.class };
    }
    
    private static class PolicyEntry
    {
        private final CodeSource codesource;
        final List<Permission> permissions;
        private final List<PolicyParser.PrincipalEntry> principals;
        
        PolicyEntry(CodeSource codesource, List<PolicyParser.PrincipalEntry> principals) {
            this.codesource = codesource;
            this.permissions = new ArrayList();
            this.principals = principals;
        }
        
        PolicyEntry(CodeSource codeSource) {
            this(codeSource, null);
        }
        
        List<PolicyParser.PrincipalEntry> getPrincipals() {
            return this.principals;
        }
        
        void add(Permission permission) {
            this.permissions.add(permission);
        }
        
        CodeSource getCodeSource() {
            return this.codesource;
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            sb.append(ResourcesMgr.getString("LPARAM"));
            sb.append(this.getCodeSource());
            sb.append("\n");
            for (int i = 0; i < this.permissions.size(); ++i) {
                Permission obj = (Permission)this.permissions.get(i);
                sb.append(ResourcesMgr.getString("SPACE"));
                sb.append(ResourcesMgr.getString("SPACE"));
                sb.append(obj);
                sb.append(ResourcesMgr.getString("NEWLINE"));
            }
            sb.append(ResourcesMgr.getString("RPARAM"));
            sb.append(ResourcesMgr.getString("NEWLINE"));
            return sb.toString();
        }
    }
    
    private static class SelfPermission extends Permission
    {
        private static final long serialVersionUID = -8315562579967246806L;
        private String type;
        private String name;
        private String actions;
        private Certificate[] certs;
        
        public SelfPermission(String s, String name, String actions, Certificate[] array) {
            super(s);
            if (s == null) {
                throw new NullPointerException(ResourcesMgr.getString("type.can.t.be.null"));
            }
            this.type = s;
            this.name = name;
            this.actions = actions;
            if (array != null) {
                int i = 0;
                while (i < array.length) {
                    if (!(array[i] instanceof X509Certificate)) {
                        this.certs = (Certificate[])array.clone();
                        break;
                    }
                    ++i;
                }
                if (this.certs == null) {
                    int j = 0;
                    int n = 0;
                    while (j < array.length) {
                        ++n;
                        while (j + 1 < array.length && ((X509Certificate)array[j]).getIssuerDN().equals(((X509Certificate)array[j + 1]).getSubjectDN())) {
                            ++j;
                        }
                        ++j;
                    }
                    if (n == array.length) {
                        this.certs = (Certificate[])array.clone();
                    }
                    if (this.certs == null) {
                        ArrayList<Object> list = new ArrayList();
                        for (int k = 0; k < array.length; ++k) {
                            list.add(array[k]);
                            while (k + 1 < array.length && ((X509Certificate)array[k]).getIssuerDN().equals(((X509Certificate)array[k + 1]).getSubjectDN())) {
                                ++k;
                            }
                        }
                        list.toArray(this.certs = new Certificate[list.size()]);
                    }
                }
            }
            return;
        }
        
        @Override
        public boolean implies(Permission permission) {
            return false;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if (!(o instanceof PolicyFile.SelfPermission)) {
                return false;
            }
            PolicyFile.SelfPermission selfPermission = (PolicyFile.SelfPermission)o;
            if (!this.type.equals(selfPermission.type) || !this.name.equals(selfPermission.name) || !this.actions.equals(selfPermission.actions)) {
                return false;
            }
            if (this.certs.length != selfPermission.certs.length) {
                return false;
            }
            int i = 0;
            while (i < this.certs.length) {
                boolean n = false;
                int j = 0;
                while (j < selfPermission.certs.length) {
                    if (this.certs[i].equals(selfPermission.certs[j])) {
                        n = true;
                        break;
                    }
                    ++j;
                }
                if (n == 0) {
                    return false;
                }
                ++i;
            }
            int k = 0;
            while (k < selfPermission.certs.length) {
                boolean n2 = false;
                int l = 0;
                while (l < this.certs.length) {
                    if (selfPermission.certs[k].equals(this.certs[l])) {
                        n2 = true;
                        break;
                    }
                    ++l;
                }
                if (n2 == 0) {
                    return false;
                }
                ++k;
            }
            return true;
        }
        
        @Override
        public int hashCode() {
            int hashCode = this.type.hashCode();
            if (this.name != null) {
                hashCode ^= this.name.hashCode();
            }
            if (this.actions != null) {
                hashCode ^= this.actions.hashCode();
            }
            return hashCode;
        }
        
        @Override
        public String getActions() {
            return "";
        }
        
        public String getSelfType() {
            return this.type;
        }
        
        public String getSelfName() {
            return this.name;
        }
        
        public String getSelfActions() {
            return this.actions;
        }
        
        public Certificate[] getCerts() {
            return this.certs;
        }
        
        @Override
        public String toString() {
            return "(SelfPermission " + this.type + " " + this.name + " " + this.actions + ")";
        }
    }
    
    private static class PolicyInfo
    {
        private static final boolean verbose = false;
        final List<PolicyFile.PolicyEntry> policyEntries;
        final List<PolicyFile.PolicyEntry> identityPolicyEntries;
        final Map<Object, Object> aliasMapping;
        private final JavaSecurityProtectionDomainAccess.ProtectionDomainCache[] pdMapping;
        private Random random;
        
        PolicyInfo(int n) {
            this.policyEntries = new ArrayList();
            this.identityPolicyEntries = Collections.synchronizedList(new ArrayList(2));
            this.aliasMapping = Collections.synchronizedMap(new HashMap(11));
            this.pdMapping = new JavaSecurityProtectionDomainAccess.ProtectionDomainCache[n];
            JavaSecurityProtectionDomainAccess javaSecurityProtectionDomainAccess = SharedSecrets.getJavaSecurityProtectionDomainAccess();
            for (int i = 0; i < n; ++i) {
                this.pdMapping[i] = javaSecurityProtectionDomainAccess.getProtectionDomainCache();
            }
            if (n > 1) {
                this.random = new Random();
            }
        }
        
        JavaSecurityProtectionDomainAccess.ProtectionDomainCache getPdMapping() {
            if (this.pdMapping.length == 1) {
                return this.pdMapping[0];
            }
            return this.pdMapping[Math.abs(this.random.nextInt() % this.pdMapping.length)];
        }
    }
}
